@@ -10,8 +10,7 @@ __all__ = [
     "cluster",
     "clustersetup",
     "node",
-    "ssh",
-    "scp",
+    "sshutils",
     "plugins",
     "balancers",
     "managers",
