@@ -30,6 +30,9 @@ class CmdSshNode(NodeCompleter):
         parser.add_option("-u", "--user", dest="user", action="store",
                           type="string", default='root',
                           help="login as USER (defaults to root)")
+        parser.add_option("-X", "--forward-x11", dest="forward_x11",
+                          action="store_true", default=False,
+                          help="enable X11 forwarding ")
 
     def execute(self, args):
         if len(args) < 2:
@@ -39,6 +42,7 @@ class CmdSshNode(NodeCompleter):
         node = args[1]
         cmd = ' '.join(args[2:])
         retval = self.cm.ssh_to_cluster_node(scluster, node,
-                                             user=self.opts.user, command=cmd)
+                                             user=self.opts.user, command=cmd,
+                                             forward_x11=self.opts.forward_x11)
         if cmd and retval is not None:
             sys.exit(retval)
